# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["RoomRecordingRetrieveResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """A unique identifier for the room recording."""

    codec: Optional[str] = None
    """Shows the codec used for the room recording."""

    completed_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room recording has completed."""

    created_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room recording was created."""

    download_url: Optional[str] = None
    """Url to download the recording."""

    duration_secs: Optional[int] = None
    """Shows the room recording duration in seconds."""

    ended_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room recording has ended."""

    participant_id: Optional[str] = None
    """Identify the room participant associated with the room recording."""

    record_type: Optional[str] = None

    room_id: Optional[str] = None
    """Identify the room associated with the room recording."""

    session_id: Optional[str] = None
    """Identify the room session associated with the room recording."""

    size_mb: Optional[float] = None
    """Shows the room recording size in MB."""

    started_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room recording has stated."""

    status: Optional[Literal["completed", "processing"]] = None
    """Shows the room recording status."""

    type: Optional[Literal["audio", "video"]] = None
    """Shows the room recording type."""

    updated_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room recording was updated."""


class RoomRecordingRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import (
    DtmfType,
    EncryptedMedia,
    AnchorsiteOverride,
    credential_connection_list_params,
    credential_connection_create_params,
    credential_connection_update_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.dtmf_type import DtmfType
from ...types.encrypted_media import EncryptedMedia
from ...types.anchorsite_override import AnchorsiteOverride
from ...types.credential_connection import CredentialConnection
from ...types.credential_inbound_param import CredentialInboundParam
from ...types.credential_outbound_param import CredentialOutboundParam
from ...types.connection_rtcp_settings_param import ConnectionRtcpSettingsParam
from ...types.credential_connection_create_response import CredentialConnectionCreateResponse
from ...types.credential_connection_delete_response import CredentialConnectionDeleteResponse
from ...types.credential_connection_update_response import CredentialConnectionUpdateResponse
from ...types.shared_params.connection_jitter_buffer import ConnectionJitterBuffer
from ...types.credential_connection_retrieve_response import CredentialConnectionRetrieveResponse
from ...types.shared_params.connection_noise_suppression_details import ConnectionNoiseSuppressionDetails

__all__ = ["CredentialConnectionsResource", "AsyncCredentialConnectionsResource"]


class CredentialConnectionsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CredentialConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CredentialConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CredentialConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CredentialConnectionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        connection_name: str,
        password: str,
        user_name: str,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: CredentialInboundParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: CredentialOutboundParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        sip_uri_calling_preference: Literal["disabled", "unrestricted", "internal"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2", "texml"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionCreateResponse:
        """
        Creates a credential connection.

        Args:
          connection_name: A user-assigned name to help manage the connection.

          password: The password to be used as part of the credentials. Must be 8 to 128 characters
              long.

          user_name: The user name to be used as part of the credentials. Must be 4-32 characters
              long and alphanumeric values only (no spaces or special characters).

          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          sip_uri_calling_preference: This feature enables inbound SIP URI calls to your Credential Auth Connection.
              If enabled for all (unrestricted) then anyone who calls the SIP URI
              <your-username>@telnyx.com will be connected to your Connection. You can also
              choose to allow only calls that are originated on any Connections under your
              account (internal).

          tags: Tags associated with the connection.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1, v2 or texml. Note -
              texml can only be set when the outbound object parameter call_parking_enabled is
              included and set to true.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/credential_connections",
            body=maybe_transform(
                {
                    "connection_name": connection_name,
                    "password": password,
                    "user_name": user_name,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "sip_uri_calling_preference": sip_uri_calling_preference,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                credential_connection_create_params.CredentialConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionRetrieveResponse:
        """
        Retrieves the details of an existing credential connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/credential_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: CredentialInboundParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: CredentialOutboundParam | Omit = omit,
        password: str | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        sip_uri_calling_preference: Literal["disabled", "unrestricted", "internal"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        user_name: str | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionUpdateResponse:
        """
        Updates settings of an existing credential connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          connection_name: A user-assigned name to help manage the connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          password: The password to be used as part of the credentials. Must be 8 to 128 characters
              long.

          sip_uri_calling_preference: This feature enables inbound SIP URI calls to your Credential Auth Connection.
              If enabled for all (unrestricted) then anyone who calls the SIP URI
              <your-username>@telnyx.com will be connected to your Connection. You can also
              choose to allow only calls that are originated on any Connections under your
              account (internal).

          tags: Tags associated with the connection.

          user_name: The user name to be used as part of the credentials. Must be 4-32 characters
              long and alphanumeric values only (no spaces or special characters).

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/credential_connections/{id}",
            body=maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "password": password,
                    "rtcp_settings": rtcp_settings,
                    "sip_uri_calling_preference": sip_uri_calling_preference,
                    "tags": tags,
                    "user_name": user_name,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                credential_connection_update_params.CredentialConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: credential_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[CredentialConnection]:
        """
        Returns a list of your credential connections.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/credential_connections",
            page=SyncDefaultFlatPagination[CredentialConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    credential_connection_list_params.CredentialConnectionListParams,
                ),
            ),
            model=CredentialConnection,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionDeleteResponse:
        """
        Deletes an existing credential connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/credential_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionDeleteResponse,
        )


class AsyncCredentialConnectionsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCredentialConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCredentialConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCredentialConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCredentialConnectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        connection_name: str,
        password: str,
        user_name: str,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: CredentialInboundParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: CredentialOutboundParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        sip_uri_calling_preference: Literal["disabled", "unrestricted", "internal"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2", "texml"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionCreateResponse:
        """
        Creates a credential connection.

        Args:
          connection_name: A user-assigned name to help manage the connection.

          password: The password to be used as part of the credentials. Must be 8 to 128 characters
              long.

          user_name: The user name to be used as part of the credentials. Must be 4-32 characters
              long and alphanumeric values only (no spaces or special characters).

          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          sip_uri_calling_preference: This feature enables inbound SIP URI calls to your Credential Auth Connection.
              If enabled for all (unrestricted) then anyone who calls the SIP URI
              <your-username>@telnyx.com will be connected to your Connection. You can also
              choose to allow only calls that are originated on any Connections under your
              account (internal).

          tags: Tags associated with the connection.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1, v2 or texml. Note -
              texml can only be set when the outbound object parameter call_parking_enabled is
              included and set to true.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/credential_connections",
            body=await async_maybe_transform(
                {
                    "connection_name": connection_name,
                    "password": password,
                    "user_name": user_name,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "sip_uri_calling_preference": sip_uri_calling_preference,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                credential_connection_create_params.CredentialConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionRetrieveResponse:
        """
        Retrieves the details of an existing credential connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/credential_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: CredentialInboundParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: CredentialOutboundParam | Omit = omit,
        password: str | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        sip_uri_calling_preference: Literal["disabled", "unrestricted", "internal"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        user_name: str | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionUpdateResponse:
        """
        Updates settings of an existing credential connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          connection_name: A user-assigned name to help manage the connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          password: The password to be used as part of the credentials. Must be 8 to 128 characters
              long.

          sip_uri_calling_preference: This feature enables inbound SIP URI calls to your Credential Auth Connection.
              If enabled for all (unrestricted) then anyone who calls the SIP URI
              <your-username>@telnyx.com will be connected to your Connection. You can also
              choose to allow only calls that are originated on any Connections under your
              account (internal).

          tags: Tags associated with the connection.

          user_name: The user name to be used as part of the credentials. Must be 4-32 characters
              long and alphanumeric values only (no spaces or special characters).

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/credential_connections/{id}",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "password": password,
                    "rtcp_settings": rtcp_settings,
                    "sip_uri_calling_preference": sip_uri_calling_preference,
                    "tags": tags,
                    "user_name": user_name,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                credential_connection_update_params.CredentialConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: credential_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CredentialConnection, AsyncDefaultFlatPagination[CredentialConnection]]:
        """
        Returns a list of your credential connections.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/credential_connections",
            page=AsyncDefaultFlatPagination[CredentialConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    credential_connection_list_params.CredentialConnectionListParams,
                ),
            ),
            model=CredentialConnection,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialConnectionDeleteResponse:
        """
        Deletes an existing credential connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/credential_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialConnectionDeleteResponse,
        )


class CredentialConnectionsResourceWithRawResponse:
    def __init__(self, credential_connections: CredentialConnectionsResource) -> None:
        self._credential_connections = credential_connections

        self.create = to_raw_response_wrapper(
            credential_connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            credential_connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            credential_connections.update,
        )
        self.list = to_raw_response_wrapper(
            credential_connections.list,
        )
        self.delete = to_raw_response_wrapper(
            credential_connections.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._credential_connections.actions)


class AsyncCredentialConnectionsResourceWithRawResponse:
    def __init__(self, credential_connections: AsyncCredentialConnectionsResource) -> None:
        self._credential_connections = credential_connections

        self.create = async_to_raw_response_wrapper(
            credential_connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            credential_connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            credential_connections.update,
        )
        self.list = async_to_raw_response_wrapper(
            credential_connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            credential_connections.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._credential_connections.actions)


class CredentialConnectionsResourceWithStreamingResponse:
    def __init__(self, credential_connections: CredentialConnectionsResource) -> None:
        self._credential_connections = credential_connections

        self.create = to_streamed_response_wrapper(
            credential_connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            credential_connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            credential_connections.update,
        )
        self.list = to_streamed_response_wrapper(
            credential_connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            credential_connections.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._credential_connections.actions)


class AsyncCredentialConnectionsResourceWithStreamingResponse:
    def __init__(self, credential_connections: AsyncCredentialConnectionsResource) -> None:
        self._credential_connections = credential_connections

        self.create = async_to_streamed_response_wrapper(
            credential_connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            credential_connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            credential_connections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            credential_connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            credential_connections.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._credential_connections.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.texml.accounts.calls import siprec_siprec_sid_json_params
from .....types.texml.accounts.calls.siprec_siprec_sid_json_response import SiprecSiprecSidJsonResponse

__all__ = ["SiprecResource", "AsyncSiprecResource"]


class SiprecResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SiprecResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SiprecResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SiprecResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SiprecResourceWithStreamingResponse(self)

    def siprec_sid_json(
        self,
        siprec_sid: str,
        *,
        account_sid: str,
        call_sid: str,
        status: Literal["stopped"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecSiprecSidJsonResponse:
        """
        Updates siprec session identified by siprec_sid.

        Args:
          status: The new status of the resource. Specifying `stopped` will end the siprec
              session.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        if not siprec_sid:
            raise ValueError(f"Expected a non-empty value for `siprec_sid` but received {siprec_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Siprec/{siprec_sid}.json",
            body=maybe_transform({"status": status}, siprec_siprec_sid_json_params.SiprecSiprecSidJsonParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecSiprecSidJsonResponse,
        )


class AsyncSiprecResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSiprecResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSiprecResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSiprecResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSiprecResourceWithStreamingResponse(self)

    async def siprec_sid_json(
        self,
        siprec_sid: str,
        *,
        account_sid: str,
        call_sid: str,
        status: Literal["stopped"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecSiprecSidJsonResponse:
        """
        Updates siprec session identified by siprec_sid.

        Args:
          status: The new status of the resource. Specifying `stopped` will end the siprec
              session.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        if not siprec_sid:
            raise ValueError(f"Expected a non-empty value for `siprec_sid` but received {siprec_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Siprec/{siprec_sid}.json",
            body=await async_maybe_transform(
                {"status": status}, siprec_siprec_sid_json_params.SiprecSiprecSidJsonParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecSiprecSidJsonResponse,
        )


class SiprecResourceWithRawResponse:
    def __init__(self, siprec: SiprecResource) -> None:
        self._siprec = siprec

        self.siprec_sid_json = to_raw_response_wrapper(
            siprec.siprec_sid_json,
        )


class AsyncSiprecResourceWithRawResponse:
    def __init__(self, siprec: AsyncSiprecResource) -> None:
        self._siprec = siprec

        self.siprec_sid_json = async_to_raw_response_wrapper(
            siprec.siprec_sid_json,
        )


class SiprecResourceWithStreamingResponse:
    def __init__(self, siprec: SiprecResource) -> None:
        self._siprec = siprec

        self.siprec_sid_json = to_streamed_response_wrapper(
            siprec.siprec_sid_json,
        )


class AsyncSiprecResourceWithStreamingResponse:
    def __init__(self, siprec: AsyncSiprecResource) -> None:
        self._siprec = siprec

        self.siprec_sid_json = async_to_streamed_response_wrapper(
            siprec.siprec_sid_json,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_siprec_failed import CallSiprecFailed

__all__ = ["CallSiprecFailedWebhookEvent"]


class CallSiprecFailedWebhookEvent(BaseModel):
    data: Optional[CallSiprecFailed] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["AutoRechargePrefUpdateParams"]


class AutoRechargePrefUpdateParams(TypedDict, total=False):
    enabled: bool
    """Whether auto recharge is enabled."""

    invoice_enabled: bool

    preference: Literal["credit_paypal", "ach"]
    """The payment preference for auto recharge."""

    recharge_amount: str
    """
    The amount to recharge the account, the actual recharge amount will be the
    amount necessary to reach the threshold amount plus the recharge amount.
    """

    threshold_amount: str
    """The threshold amount at which the account will be recharged."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import alphanumeric_sender_id_list_params, alphanumeric_sender_id_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.alphanumeric_sender_id_list_response import AlphanumericSenderIDListResponse
from ..types.alphanumeric_sender_id_create_response import AlphanumericSenderIDCreateResponse
from ..types.alphanumeric_sender_id_delete_response import AlphanumericSenderIDDeleteResponse
from ..types.alphanumeric_sender_id_retrieve_response import AlphanumericSenderIDRetrieveResponse

__all__ = ["AlphanumericSenderIDsResource", "AsyncAlphanumericSenderIDsResource"]


class AlphanumericSenderIDsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AlphanumericSenderIDsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AlphanumericSenderIDsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AlphanumericSenderIDsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AlphanumericSenderIDsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        alphanumeric_sender_id: str,
        messaging_profile_id: str,
        us_long_code_fallback: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlphanumericSenderIDCreateResponse:
        """
        Create a new alphanumeric sender ID associated with a messaging profile.

        Args:
          alphanumeric_sender_id: The alphanumeric sender ID string.

          messaging_profile_id: The messaging profile to associate the sender ID with.

          us_long_code_fallback: A US long code number to use as fallback when sending to US destinations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/alphanumeric_sender_ids",
            body=maybe_transform(
                {
                    "alphanumeric_sender_id": alphanumeric_sender_id,
                    "messaging_profile_id": messaging_profile_id,
                    "us_long_code_fallback": us_long_code_fallback,
                },
                alphanumeric_sender_id_create_params.AlphanumericSenderIDCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlphanumericSenderIDCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlphanumericSenderIDRetrieveResponse:
        """
        Retrieve a specific alphanumeric sender ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/alphanumeric_sender_ids/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlphanumericSenderIDRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_messaging_profile_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[AlphanumericSenderIDListResponse]:
        """
        List all alphanumeric sender IDs for the authenticated user.

        Args:
          filter_messaging_profile_id: Filter by messaging profile ID.

          page_number: Page number.

          page_size: Page size.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/alphanumeric_sender_ids",
            page=SyncDefaultFlatPagination[AlphanumericSenderIDListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_messaging_profile_id": filter_messaging_profile_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    alphanumeric_sender_id_list_params.AlphanumericSenderIDListParams,
                ),
            ),
            model=AlphanumericSenderIDListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlphanumericSenderIDDeleteResponse:
        """
        Delete an alphanumeric sender ID and disassociate it from its messaging profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/alphanumeric_sender_ids/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlphanumericSenderIDDeleteResponse,
        )


class AsyncAlphanumericSenderIDsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAlphanumericSenderIDsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAlphanumericSenderIDsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAlphanumericSenderIDsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAlphanumericSenderIDsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        alphanumeric_sender_id: str,
        messaging_profile_id: str,
        us_long_code_fallback: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlphanumericSenderIDCreateResponse:
        """
        Create a new alphanumeric sender ID associated with a messaging profile.

        Args:
          alphanumeric_sender_id: The alphanumeric sender ID string.

          messaging_profile_id: The messaging profile to associate the sender ID with.

          us_long_code_fallback: A US long code number to use as fallback when sending to US destinations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/alphanumeric_sender_ids",
            body=await async_maybe_transform(
                {
                    "alphanumeric_sender_id": alphanumeric_sender_id,
                    "messaging_profile_id": messaging_profile_id,
                    "us_long_code_fallback": us_long_code_fallback,
                },
                alphanumeric_sender_id_create_params.AlphanumericSenderIDCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlphanumericSenderIDCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlphanumericSenderIDRetrieveResponse:
        """
        Retrieve a specific alphanumeric sender ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/alphanumeric_sender_ids/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlphanumericSenderIDRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_messaging_profile_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AlphanumericSenderIDListResponse, AsyncDefaultFlatPagination[AlphanumericSenderIDListResponse]]:
        """
        List all alphanumeric sender IDs for the authenticated user.

        Args:
          filter_messaging_profile_id: Filter by messaging profile ID.

          page_number: Page number.

          page_size: Page size.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/alphanumeric_sender_ids",
            page=AsyncDefaultFlatPagination[AlphanumericSenderIDListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_messaging_profile_id": filter_messaging_profile_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    alphanumeric_sender_id_list_params.AlphanumericSenderIDListParams,
                ),
            ),
            model=AlphanumericSenderIDListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlphanumericSenderIDDeleteResponse:
        """
        Delete an alphanumeric sender ID and disassociate it from its messaging profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/alphanumeric_sender_ids/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlphanumericSenderIDDeleteResponse,
        )


class AlphanumericSenderIDsResourceWithRawResponse:
    def __init__(self, alphanumeric_sender_ids: AlphanumericSenderIDsResource) -> None:
        self._alphanumeric_sender_ids = alphanumeric_sender_ids

        self.create = to_raw_response_wrapper(
            alphanumeric_sender_ids.create,
        )
        self.retrieve = to_raw_response_wrapper(
            alphanumeric_sender_ids.retrieve,
        )
        self.list = to_raw_response_wrapper(
            alphanumeric_sender_ids.list,
        )
        self.delete = to_raw_response_wrapper(
            alphanumeric_sender_ids.delete,
        )


class AsyncAlphanumericSenderIDsResourceWithRawResponse:
    def __init__(self, alphanumeric_sender_ids: AsyncAlphanumericSenderIDsResource) -> None:
        self._alphanumeric_sender_ids = alphanumeric_sender_ids

        self.create = async_to_raw_response_wrapper(
            alphanumeric_sender_ids.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            alphanumeric_sender_ids.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            alphanumeric_sender_ids.list,
        )
        self.delete = async_to_raw_response_wrapper(
            alphanumeric_sender_ids.delete,
        )


class AlphanumericSenderIDsResourceWithStreamingResponse:
    def __init__(self, alphanumeric_sender_ids: AlphanumericSenderIDsResource) -> None:
        self._alphanumeric_sender_ids = alphanumeric_sender_ids

        self.create = to_streamed_response_wrapper(
            alphanumeric_sender_ids.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            alphanumeric_sender_ids.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            alphanumeric_sender_ids.list,
        )
        self.delete = to_streamed_response_wrapper(
            alphanumeric_sender_ids.delete,
        )


class AsyncAlphanumericSenderIDsResourceWithStreamingResponse:
    def __init__(self, alphanumeric_sender_ids: AsyncAlphanumericSenderIDsResource) -> None:
        self._alphanumeric_sender_ids = alphanumeric_sender_ids

        self.create = async_to_streamed_response_wrapper(
            alphanumeric_sender_ids.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            alphanumeric_sender_ids.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            alphanumeric_sender_ids.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            alphanumeric_sender_ids.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SubNumberOrderListResponse,
    SubNumberOrderCancelResponse,
    SubNumberOrderUpdateResponse,
    SubNumberOrderRetrieveResponse,
    SubNumberOrderUpdateRequirementGroupResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSubNumberOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.retrieve(
            sub_number_order_id="sub_number_order_id",
        )
        assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.retrieve(
            sub_number_order_id="sub_number_order_id",
            filter={"include_phone_numbers": True},
        )
        assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sub_number_orders.with_raw_response.retrieve(
            sub_number_order_id="sub_number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = response.parse()
        assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sub_number_orders.with_streaming_response.retrieve(
            sub_number_order_id="sub_number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = response.parse()
            assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sub_number_order_id` but received ''"):
            client.sub_number_orders.with_raw_response.retrieve(
                sub_number_order_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.update(
            sub_number_order_id="sub_number_order_id",
        )
        assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.update(
            sub_number_order_id="sub_number_order_id",
            regulatory_requirements=[
                {
                    "field_value": "45f45a04-b4be-4592-95b1-9306b9db2b21",
                    "requirement_id": "8ffb3622-7c6b-4ccc-b65f-7a3dc0099576",
                }
            ],
        )
        assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.sub_number_orders.with_raw_response.update(
            sub_number_order_id="sub_number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = response.parse()
        assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.sub_number_orders.with_streaming_response.update(
            sub_number_order_id="sub_number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = response.parse()
            assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sub_number_order_id` but received ''"):
            client.sub_number_orders.with_raw_response.update(
                sub_number_order_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.list()
        assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.list(
            filter={
                "country_code": "US",
                "order_request_id": "12ade33a-21c0-473b-b055-b3c836e1c293",
                "phone_number_type": "local",
                "phone_numbers_count": 1,
                "status": "status",
            },
        )
        assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.sub_number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = response.parse()
        assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.sub_number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = response.parse()
            assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_cancel(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.cancel(
            "sub_number_order_id",
        )
        assert_matches_type(SubNumberOrderCancelResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_cancel(self, client: Telnyx) -> None:
        response = client.sub_number_orders.with_raw_response.cancel(
            "sub_number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = response.parse()
        assert_matches_type(SubNumberOrderCancelResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_cancel(self, client: Telnyx) -> None:
        with client.sub_number_orders.with_streaming_response.cancel(
            "sub_number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = response.parse()
            assert_matches_type(SubNumberOrderCancelResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_cancel(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sub_number_order_id` but received ''"):
            client.sub_number_orders.with_raw_response.cancel(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_requirement_group(self, client: Telnyx) -> None:
        sub_number_order = client.sub_number_orders.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
        )
        assert_matches_type(SubNumberOrderUpdateRequirementGroupResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_requirement_group(self, client: Telnyx) -> None:
        response = client.sub_number_orders.with_raw_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = response.parse()
        assert_matches_type(SubNumberOrderUpdateRequirementGroupResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_requirement_group(self, client: Telnyx) -> None:
        with client.sub_number_orders.with_streaming_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = response.parse()
            assert_matches_type(SubNumberOrderUpdateRequirementGroupResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_requirement_group(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sub_number_orders.with_raw_response.update_requirement_group(
                id="",
                requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
            )


class TestAsyncSubNumberOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.retrieve(
            sub_number_order_id="sub_number_order_id",
        )
        assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.retrieve(
            sub_number_order_id="sub_number_order_id",
            filter={"include_phone_numbers": True},
        )
        assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders.with_raw_response.retrieve(
            sub_number_order_id="sub_number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = await response.parse()
        assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders.with_streaming_response.retrieve(
            sub_number_order_id="sub_number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = await response.parse()
            assert_matches_type(SubNumberOrderRetrieveResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sub_number_order_id` but received ''"):
            await async_client.sub_number_orders.with_raw_response.retrieve(
                sub_number_order_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.update(
            sub_number_order_id="sub_number_order_id",
        )
        assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.update(
            sub_number_order_id="sub_number_order_id",
            regulatory_requirements=[
                {
                    "field_value": "45f45a04-b4be-4592-95b1-9306b9db2b21",
                    "requirement_id": "8ffb3622-7c6b-4ccc-b65f-7a3dc0099576",
                }
            ],
        )
        assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders.with_raw_response.update(
            sub_number_order_id="sub_number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = await response.parse()
        assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders.with_streaming_response.update(
            sub_number_order_id="sub_number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = await response.parse()
            assert_matches_type(SubNumberOrderUpdateResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sub_number_order_id` but received ''"):
            await async_client.sub_number_orders.with_raw_response.update(
                sub_number_order_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.list()
        assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.list(
            filter={
                "country_code": "US",
                "order_request_id": "12ade33a-21c0-473b-b055-b3c836e1c293",
                "phone_number_type": "local",
                "phone_numbers_count": 1,
                "status": "status",
            },
        )
        assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = await response.parse()
        assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = await response.parse()
            assert_matches_type(SubNumberOrderListResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_cancel(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.cancel(
            "sub_number_order_id",
        )
        assert_matches_type(SubNumberOrderCancelResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders.with_raw_response.cancel(
            "sub_number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = await response.parse()
        assert_matches_type(SubNumberOrderCancelResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders.with_streaming_response.cancel(
            "sub_number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = await response.parse()
            assert_matches_type(SubNumberOrderCancelResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sub_number_order_id` but received ''"):
            await async_client.sub_number_orders.with_raw_response.cancel(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        sub_number_order = await async_client.sub_number_orders.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
        )
        assert_matches_type(SubNumberOrderUpdateRequirementGroupResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders.with_raw_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_order = await response.parse()
        assert_matches_type(SubNumberOrderUpdateRequirementGroupResponse, sub_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders.with_streaming_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_order = await response.parse()
            assert_matches_type(SubNumberOrderUpdateRequirementGroupResponse, sub_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sub_number_orders.with_raw_response.update_requirement_group(
                id="",
                requirement_group_id="a4b201f9-8646-4e54-a7d2-b2e403eeaf8c",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["DeepgramNova3ConfigParam"]


class DeepgramNova3ConfigParam(TypedDict, total=False):
    transcription_engine: Required[Literal["Deepgram"]]

    transcription_model: Required[Literal["deepgram/nova-3"]]

    interim_results: bool
    """Whether to send also interim results.

    If set to false, only final results will be sent.
    """

    keywords_boosting: Dict[str, float]
    """
    Keywords and their respective intensifiers (boosting values) to improve
    transcription accuracy for specific words or phrases. The intensifier should be
    a numeric value. Example: `{"snuffleupagus": 5, "systrom": 2, "krieger": 1}`.
    """

    language: Literal[
        "en-US",
        "en-AU",
        "en-GB",
        "en-IN",
        "en-NZ",
        "de",
        "nl",
        "sv-SE",
        "da-DK",
        "es",
        "es-419",
        "fr",
        "fr-CA",
        "pt-BR",
        "pt-PT",
        "auto_detect",
    ]
    """Language to use for speech recognition with nova-3 model"""

    utterance_end_ms: int
    """Number of milliseconds of silence to consider an utterance ended.

    Ranges from 0 to 5000 ms.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .transcription import Transcription

__all__ = ["TranscriptionWebhookEvent"]


class TranscriptionWebhookEvent(BaseModel):
    data: Optional[Transcription] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .assistants.tests.test_suites.meta import Meta

__all__ = ["ConversationRetrieveConversationsInsightsResponse", "Data", "DataConversationInsight"]


class DataConversationInsight(BaseModel):
    insight_id: str
    """Unique identifier for the insight configuration."""

    result: str
    """Insight result from the conversation.

    If the insight has a JSON schema, this will be stringified JSON object.
    """


class Data(BaseModel):
    id: str
    """Unique identifier for the conversation insight."""

    conversation_insights: List[DataConversationInsight]
    """List of insights extracted from the conversation."""

    created_at: datetime
    """Timestamp of when the object was created."""

    status: Literal["pending", "in_progress", "completed", "failed"]
    """Current status of the insight generation for the conversation."""


class ConversationRetrieveConversationsInsightsResponse(BaseModel):
    data: List[Data]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CommentCreateResponse", "Data"]


class Data(BaseModel):
    id: str

    body: str
    """Comment body"""

    created_at: str
    """Comment creation timestamp in ISO 8601 format"""

    user_id: str
    """Identifies the user who created the comment.

    Will be null if created by Telnyx Admin
    """

    portout_id: Optional[str] = None
    """Identifies the associated port request"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class CommentCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VirtualCrossConnectCreateParams"]


class VirtualCrossConnectCreateParams(TypedDict, total=False):
    region_code: Required[str]
    """The region the interface should be deployed to."""

    bandwidth_mbps: float
    """
    The desired throughput in Megabits per Second (Mbps) for your Virtual Cross
    Connect.<br /><br />The available bandwidths can be found using the
    /virtual_cross_connect_regions endpoint.
    """

    bgp_asn: float
    """The Border Gateway Protocol (BGP) Autonomous System Number (ASN).

    If null, value will be assigned by Telnyx.
    """

    cloud_provider: Literal["aws", "azure", "gce"]
    """
    The Virtual Private Cloud with which you would like to establish a cross
    connect.
    """

    cloud_provider_region: str
    """
    The region where your Virtual Private Cloud hosts are located.<br /><br />The
    available regions can be found using the /virtual_cross_connect_regions
    endpoint.
    """

    name: str
    """A user specified name for the interface."""

    network_id: str
    """The id of the network associated with the interface."""

    primary_bgp_key: str
    """The authentication key for BGP peer configuration."""

    primary_cloud_account_id: str
    """The identifier for your Virtual Private Cloud.

    The number will be different based upon your Cloud provider.
    """

    primary_cloud_ip: str
    """
    The IP address assigned for your side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value should be null for GCE as Google will only inform you
    of your assigned IP once the connection has been accepted.
    """

    primary_telnyx_ip: str
    """
    The IP address assigned to the Telnyx side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value should be null for GCE as Google will only inform you
    of your assigned IP once the connection has been accepted.
    """

    secondary_bgp_key: str
    """The authentication key for BGP peer configuration."""

    secondary_cloud_account_id: str
    """The identifier for your Virtual Private Cloud.

    The number will be different based upon your Cloud provider.<br /><br />This
    attribute is only necessary for GCE.
    """

    secondary_cloud_ip: str
    """
    The IP address assigned for your side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value should be null for GCE as Google will only inform you
    of your assigned IP once the connection has been accepted.
    """

    secondary_telnyx_ip: str
    """
    The IP address assigned to the Telnyx side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value should be null for GCE as Google will only inform you
    of your assigned IP once the connection has been accepted.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..shared.messaging_hosted_number_order import MessagingHostedNumberOrder

__all__ = ["ActionUploadFileResponse"]


class ActionUploadFileResponse(BaseModel):
    data: Optional[MessagingHostedNumberOrder] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel
from .cdr_detailed_req_response import CdrDetailedReqResponse

__all__ = ["VoiceCreateResponse"]


class VoiceCreateResponse(BaseModel):
    data: Optional[CdrDetailedReqResponse] = None
    """Response object for CDR detailed report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["GcsConfigurationData"]


class GcsConfigurationData(BaseModel):
    backend: Literal["gcs"]
    """Storage backend type"""

    bucket: Optional[str] = None
    """Name of the bucket to be used to store recording files."""

    credentials: Optional[str] = None
    """
    Opaque credential token used to authenticate and authorize with storage
    provider.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["NetworkCreateParams"]


class NetworkCreateParams(TypedDict, total=False):
    name: Required[str]
    """A user specified name for the network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .s3_configuration_data_param import S3ConfigurationDataParam
from .gcs_configuration_data_param import GcsConfigurationDataParam
from .azure_configuration_data_param import AzureConfigurationDataParam

__all__ = ["CustomStorageCredentialCreateParams", "Configuration"]


class CustomStorageCredentialCreateParams(TypedDict, total=False):
    backend: Required[Literal["gcs", "s3", "azure"]]

    configuration: Required[Configuration]


Configuration: TypeAlias = Union[GcsConfigurationDataParam, S3ConfigurationDataParam, AzureConfigurationDataParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_streaming_stopped import CallStreamingStopped

__all__ = ["CallStreamingStoppedWebhookEvent"]


class CallStreamingStoppedWebhookEvent(BaseModel):
    data: Optional[CallStreamingStopped] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .brand.brand import (
    BrandResource,
    AsyncBrandResource,
    BrandResourceWithRawResponse,
    AsyncBrandResourceWithRawResponse,
    BrandResourceWithStreamingResponse,
    AsyncBrandResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .campaign.campaign import (
    CampaignResource,
    AsyncCampaignResource,
    CampaignResourceWithRawResponse,
    AsyncCampaignResourceWithRawResponse,
    CampaignResourceWithStreamingResponse,
    AsyncCampaignResourceWithStreamingResponse,
)
from .partner_campaigns import (
    PartnerCampaignsResource,
    AsyncPartnerCampaignsResource,
    PartnerCampaignsResourceWithRawResponse,
    AsyncPartnerCampaignsResourceWithRawResponse,
    PartnerCampaignsResourceWithStreamingResponse,
    AsyncPartnerCampaignsResourceWithStreamingResponse,
)
from .phone_number_campaigns import (
    PhoneNumberCampaignsResource,
    AsyncPhoneNumberCampaignsResource,
    PhoneNumberCampaignsResourceWithRawResponse,
    AsyncPhoneNumberCampaignsResourceWithRawResponse,
    PhoneNumberCampaignsResourceWithStreamingResponse,
    AsyncPhoneNumberCampaignsResourceWithStreamingResponse,
)
from .campaign_builder.campaign_builder import (
    CampaignBuilderResource,
    AsyncCampaignBuilderResource,
    CampaignBuilderResourceWithRawResponse,
    AsyncCampaignBuilderResourceWithRawResponse,
    CampaignBuilderResourceWithStreamingResponse,
    AsyncCampaignBuilderResourceWithStreamingResponse,
)
from .phone_number_assignment_by_profile import (
    PhoneNumberAssignmentByProfileResource,
    AsyncPhoneNumberAssignmentByProfileResource,
    PhoneNumberAssignmentByProfileResourceWithRawResponse,
    AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse,
    PhoneNumberAssignmentByProfileResourceWithStreamingResponse,
    AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse,
)
from ...types.messaging_10dlc_get_enum_response import Messaging10dlcGetEnumResponse

__all__ = ["Messaging10dlcResource", "AsyncMessaging10dlcResource"]


class Messaging10dlcResource(SyncAPIResource):
    @cached_property
    def brand(self) -> BrandResource:
        return BrandResource(self._client)

    @cached_property
    def campaign(self) -> CampaignResource:
        return CampaignResource(self._client)

    @cached_property
    def campaign_builder(self) -> CampaignBuilderResource:
        return CampaignBuilderResource(self._client)

    @cached_property
    def partner_campaigns(self) -> PartnerCampaignsResource:
        return PartnerCampaignsResource(self._client)

    @cached_property
    def phone_number_campaigns(self) -> PhoneNumberCampaignsResource:
        return PhoneNumberCampaignsResource(self._client)

    @cached_property
    def phone_number_assignment_by_profile(self) -> PhoneNumberAssignmentByProfileResource:
        return PhoneNumberAssignmentByProfileResource(self._client)

    @cached_property
    def with_raw_response(self) -> Messaging10dlcResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return Messaging10dlcResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Messaging10dlcResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return Messaging10dlcResourceWithStreamingResponse(self)

    def get_enum(
        self,
        endpoint: Literal[
            "mno",
            "optionalAttributes",
            "usecase",
            "vertical",
            "altBusinessIdType",
            "brandIdentityStatus",
            "brandRelationship",
            "campaignStatus",
            "entityType",
            "extVettingProvider",
            "vettingStatus",
            "brandStatus",
            "operationStatus",
            "approvedPublicCompany",
            "stockExchange",
            "vettingClass",
        ],
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Messaging10dlcGetEnumResponse:
        """
        Get Enum

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return cast(
            Messaging10dlcGetEnumResponse,
            self._get(
                f"/10dlc/enum/{endpoint}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, Messaging10dlcGetEnumResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncMessaging10dlcResource(AsyncAPIResource):
    @cached_property
    def brand(self) -> AsyncBrandResource:
        return AsyncBrandResource(self._client)

    @cached_property
    def campaign(self) -> AsyncCampaignResource:
        return AsyncCampaignResource(self._client)

    @cached_property
    def campaign_builder(self) -> AsyncCampaignBuilderResource:
        return AsyncCampaignBuilderResource(self._client)

    @cached_property
    def partner_campaigns(self) -> AsyncPartnerCampaignsResource:
        return AsyncPartnerCampaignsResource(self._client)

    @cached_property
    def phone_number_campaigns(self) -> AsyncPhoneNumberCampaignsResource:
        return AsyncPhoneNumberCampaignsResource(self._client)

    @cached_property
    def phone_number_assignment_by_profile(self) -> AsyncPhoneNumberAssignmentByProfileResource:
        return AsyncPhoneNumberAssignmentByProfileResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMessaging10dlcResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessaging10dlcResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessaging10dlcResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessaging10dlcResourceWithStreamingResponse(self)

    async def get_enum(
        self,
        endpoint: Literal[
            "mno",
            "optionalAttributes",
            "usecase",
            "vertical",
            "altBusinessIdType",
            "brandIdentityStatus",
            "brandRelationship",
            "campaignStatus",
            "entityType",
            "extVettingProvider",
            "vettingStatus",
            "brandStatus",
            "operationStatus",
            "approvedPublicCompany",
            "stockExchange",
            "vettingClass",
        ],
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Messaging10dlcGetEnumResponse:
        """
        Get Enum

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return cast(
            Messaging10dlcGetEnumResponse,
            await self._get(
                f"/10dlc/enum/{endpoint}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, Messaging10dlcGetEnumResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class Messaging10dlcResourceWithRawResponse:
    def __init__(self, messaging_10dlc: Messaging10dlcResource) -> None:
        self._messaging_10dlc = messaging_10dlc

        self.get_enum = to_raw_response_wrapper(
            messaging_10dlc.get_enum,
        )

    @cached_property
    def brand(self) -> BrandResourceWithRawResponse:
        return BrandResourceWithRawResponse(self._messaging_10dlc.brand)

    @cached_property
    def campaign(self) -> CampaignResourceWithRawResponse:
        return CampaignResourceWithRawResponse(self._messaging_10dlc.campaign)

    @cached_property
    def campaign_builder(self) -> CampaignBuilderResourceWithRawResponse:
        return CampaignBuilderResourceWithRawResponse(self._messaging_10dlc.campaign_builder)

    @cached_property
    def partner_campaigns(self) -> PartnerCampaignsResourceWithRawResponse:
        return PartnerCampaignsResourceWithRawResponse(self._messaging_10dlc.partner_campaigns)

    @cached_property
    def phone_number_campaigns(self) -> PhoneNumberCampaignsResourceWithRawResponse:
        return PhoneNumberCampaignsResourceWithRawResponse(self._messaging_10dlc.phone_number_campaigns)

    @cached_property
    def phone_number_assignment_by_profile(self) -> PhoneNumberAssignmentByProfileResourceWithRawResponse:
        return PhoneNumberAssignmentByProfileResourceWithRawResponse(
            self._messaging_10dlc.phone_number_assignment_by_profile
        )


class AsyncMessaging10dlcResourceWithRawResponse:
    def __init__(self, messaging_10dlc: AsyncMessaging10dlcResource) -> None:
        self._messaging_10dlc = messaging_10dlc

        self.get_enum = async_to_raw_response_wrapper(
            messaging_10dlc.get_enum,
        )

    @cached_property
    def brand(self) -> AsyncBrandResourceWithRawResponse:
        return AsyncBrandResourceWithRawResponse(self._messaging_10dlc.brand)

    @cached_property
    def campaign(self) -> AsyncCampaignResourceWithRawResponse:
        return AsyncCampaignResourceWithRawResponse(self._messaging_10dlc.campaign)

    @cached_property
    def campaign_builder(self) -> AsyncCampaignBuilderResourceWithRawResponse:
        return AsyncCampaignBuilderResourceWithRawResponse(self._messaging_10dlc.campaign_builder)

    @cached_property
    def partner_campaigns(self) -> AsyncPartnerCampaignsResourceWithRawResponse:
        return AsyncPartnerCampaignsResourceWithRawResponse(self._messaging_10dlc.partner_campaigns)

    @cached_property
    def phone_number_campaigns(self) -> AsyncPhoneNumberCampaignsResourceWithRawResponse:
        return AsyncPhoneNumberCampaignsResourceWithRawResponse(self._messaging_10dlc.phone_number_campaigns)

    @cached_property
    def phone_number_assignment_by_profile(self) -> AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse:
        return AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse(
            self._messaging_10dlc.phone_number_assignment_by_profile
        )


class Messaging10dlcResourceWithStreamingResponse:
    def __init__(self, messaging_10dlc: Messaging10dlcResource) -> None:
        self._messaging_10dlc = messaging_10dlc

        self.get_enum = to_streamed_response_wrapper(
            messaging_10dlc.get_enum,
        )

    @cached_property
    def brand(self) -> BrandResourceWithStreamingResponse:
        return BrandResourceWithStreamingResponse(self._messaging_10dlc.brand)

    @cached_property
    def campaign(self) -> CampaignResourceWithStreamingResponse:
        return CampaignResourceWithStreamingResponse(self._messaging_10dlc.campaign)

    @cached_property
    def campaign_builder(self) -> CampaignBuilderResourceWithStreamingResponse:
        return CampaignBuilderResourceWithStreamingResponse(self._messaging_10dlc.campaign_builder)

    @cached_property
    def partner_campaigns(self) -> PartnerCampaignsResourceWithStreamingResponse:
        return PartnerCampaignsResourceWithStreamingResponse(self._messaging_10dlc.partner_campaigns)

    @cached_property
    def phone_number_campaigns(self) -> PhoneNumberCampaignsResourceWithStreamingResponse:
        return PhoneNumberCampaignsResourceWithStreamingResponse(self._messaging_10dlc.phone_number_campaigns)

    @cached_property
    def phone_number_assignment_by_profile(self) -> PhoneNumberAssignmentByProfileResourceWithStreamingResponse:
        return PhoneNumberAssignmentByProfileResourceWithStreamingResponse(
            self._messaging_10dlc.phone_number_assignment_by_profile
        )


class AsyncMessaging10dlcResourceWithStreamingResponse:
    def __init__(self, messaging_10dlc: AsyncMessaging10dlcResource) -> None:
        self._messaging_10dlc = messaging_10dlc

        self.get_enum = async_to_streamed_response_wrapper(
            messaging_10dlc.get_enum,
        )

    @cached_property
    def brand(self) -> AsyncBrandResourceWithStreamingResponse:
        return AsyncBrandResourceWithStreamingResponse(self._messaging_10dlc.brand)

    @cached_property
    def campaign(self) -> AsyncCampaignResourceWithStreamingResponse:
        return AsyncCampaignResourceWithStreamingResponse(self._messaging_10dlc.campaign)

    @cached_property
    def campaign_builder(self) -> AsyncCampaignBuilderResourceWithStreamingResponse:
        return AsyncCampaignBuilderResourceWithStreamingResponse(self._messaging_10dlc.campaign_builder)

    @cached_property
    def partner_campaigns(self) -> AsyncPartnerCampaignsResourceWithStreamingResponse:
        return AsyncPartnerCampaignsResourceWithStreamingResponse(self._messaging_10dlc.partner_campaigns)

    @cached_property
    def phone_number_campaigns(self) -> AsyncPhoneNumberCampaignsResourceWithStreamingResponse:
        return AsyncPhoneNumberCampaignsResourceWithStreamingResponse(self._messaging_10dlc.phone_number_campaigns)

    @cached_property
    def phone_number_assignment_by_profile(self) -> AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse:
        return AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse(
            self._messaging_10dlc.phone_number_assignment_by_profile
        )
